@@ -1,8 +1,9 @@
 /*
  * Copyright (C) 2010 Pierre-Luc Beaudoin <pierre-luc@pierlux.com>
- * Copyright (C) 2010 Collabora
+ * Copyright (C) 2010 Collabora Ltd
  * @author Pierre-Luc Beaudoin <pierre-luc@pierlux.com>
  * @author Alban Crequy <alban.crequy@collabora.co.uk>
+ * @author Cosimo Alfarano <cosimo.alfarano@collabora.co.uk>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,13 +20,18 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 #include "config.h"
-
 #include "map-buddy-window.h"
+
+#include "conf.h"
+#include "poi-layer.h"
+#include "poi-manager.h"
+#include "poi-osm-fetcher.h"
 #include "me-marker.h"
 #include "sexy-icon-entry.h"
 #include "search.h"
 #include "merchant.h"
 #include "merchant-marker.h"
+#include "tracking.h"
 
 #include <gtk/gtk.h>
 #include <champlain-gtk/champlain-gtk.h>
@@ -34,12 +40,15 @@
 #include <gdk/gdkx.h>
 #include <X11/Xatom.h>
 #include <dbus/dbus-glib.h>
+#include <libosso-abook/osso-abook.h>
+#include <hildon/hildon-fm.h>
 
 #include <location/location-gps-device.h>
 #include <location/location-gpsd-control.h>
 #include <telepathy-glib/account.h>
 #include <telepathy-glib/contact.h>
 
+
 static void create_map_area (MapBuddyWindow *self);
 static void create_menu (MapBuddyWindow *self);
 static void create_toolbar (MapBuddyWindow *self);
@@ -47,12 +56,17 @@ static GtkWidget * create_map_selector (void);
 static GtkWidget * create_search_selector (void);
 
 static void on_copy_link (GtkButton *button, MapBuddyWindow *self);
+static void on_toggle_tracking (GtkButton *button, MapBuddyWindow *self);
 static void on_about (GtkButton *button, MapBuddyWindow *self);
 static void on_locate_me (GtkButton *button, MapBuddyWindow *self);
 static void on_layer (GtkButton *button, MapBuddyWindow *self);
 static void on_show_contacts (GtkButton *button, MapBuddyWindow *self);
+static void on_show_pois (GtkButton *button, MapBuddyWindow *self);
+static void on_get_pois (GtkButton *button, MapBuddyWindow *self);
+
 static gboolean on_key_pressed (GtkWindow* window, GdkEventKey *event, MapBuddyWindow *self);
 static void on_state_changed (GObject *gobject, GParamSpec *pspec, MapBuddyWindow *self);
+
 static void on_zoom_level_changed (GObject *gobject, GParamSpec *pspec, MapBuddyWindow *self);
 
 static void on_location_error (LocationGPSDControl *control, LocationGPSDControlError error, MapBuddyWindow *self);
@@ -98,24 +112,33 @@ map_buddy_window_dispose (GObject *object)
       self->map_buddy_contact_layer = NULL;
     }
 
+  if (self->map_buddy_poi_layer)
+    {
+      g_object_unref (self->map_buddy_poi_layer);
+      self->map_buddy_poi_layer = NULL;
+    }
+
+  if (self->poi_manager != NULL)
+    {
+      g_object_unref (self->poi_manager);
+      self->poi_manager = NULL;
+    }
+
+  if (self->conf != NULL)
+    {
+      g_object_unref (self->conf);
+      self->conf = NULL;
+    }
+
   G_OBJECT_CLASS (map_buddy_window_parent_class)->dispose (object);
 }
 
-
-static void
-map_buddy_window_finalize (GObject *object)
-{
-  G_OBJECT_CLASS (map_buddy_window_parent_class)->finalize (object);
-}
-
-
 static void
 map_buddy_window_class_init (MapBuddyWindowClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   object_class->dispose = map_buddy_window_dispose;
-  object_class->finalize = map_buddy_window_finalize;
 }
 
 
@@ -135,6 +158,11 @@ map_buddy_window_init (MapBuddyWindow *self)
   self->proxy = NULL;
   self->call = NULL;
   self->merchants = NULL;
+  self->tracking = NULL;
+
+  /* keep a reference to the conf signleton */
+  self->conf = map_buddy_conf_dup ();
+  self->poi_manager = map_buddy_poi_manager_dup ();
 
   g_signal_connect(G_OBJECT (self), "realize",
                    G_CALLBACK (on_window_realize), self);
@@ -145,6 +173,7 @@ map_buddy_window_init (MapBuddyWindow *self)
 
   /* Watch contacts location */
   self->map_buddy_contact_layer = map_buddy_contact_layer_new ();
+  self->map_buddy_poi_layer = map_buddy_poi_layer_new ();
 
   /* Create all necessary parts: */
   create_map_area (self);
@@ -223,7 +252,14 @@ create_map_area (MapBuddyWindow *self)
   /* Contact layer */
   self->contact_layer = champlain_selection_layer_new ();
   champlain_view_add_layer (self->view, CHAMPLAIN_LAYER (self->contact_layer));
-  map_buddy_contact_layer_start (self->map_buddy_contact_layer, self->contact_layer);
+  map_buddy_contact_layer_start (self->map_buddy_contact_layer,
+      self->contact_layer, GTK_WINDOW (self));
+
+  /* POI Layer */
+  self->champlain_poi_layer = champlain_selection_layer_new ();
+  champlain_view_add_layer (self->view, CHAMPLAIN_LAYER (self->champlain_poi_layer));
+  map_buddy_poi_layer_start (self->map_buddy_poi_layer, self->champlain_poi_layer,
+      GTK_WINDOW (self));
 
   self->merchant_layer = champlain_selection_layer_new ();
   champlain_view_add_layer (self->view, CHAMPLAIN_LAYER (self->merchant_layer));
@@ -509,6 +545,15 @@ create_menu (MapBuddyWindow *self)
   self->show_contacts = add_menu_item (main_menu, _("Hide contacts"),
       G_CALLBACK (on_show_contacts), self);
 
+  self->tracking_item = add_menu_item (main_menu, _("Enable Tracking"),
+      G_CALLBACK (on_toggle_tracking), self);
+
+  self->show_pois = add_menu_item (main_menu, _("Show POIs"),
+      G_CALLBACK (on_show_pois), self);
+
+  self->get_pois = add_menu_item (main_menu, _("Download POIs"),
+      G_CALLBACK (on_get_pois), self);
+
   add_menu_item (main_menu, _("Donate"),
       G_CALLBACK (open_url), "http://www.pierlux.com/map-buddy/donate/");
 
@@ -569,7 +614,9 @@ on_about (GtkButton *button,
   text = g_strdup_printf ("<big><b>%s %s</b></big>\n\n%s\n\n<small>%s</small>",
       _("Map Buddy"),
       VERSION,
-      _("Copyright \xc2\xa9 2009 Pierre-Luc Beaudoin"),
+      _("Copyright \xc2\xa9 2009-2010 Pierre-Luc Beaudoin\n"
+         "Copyright \xc2\xa9 2010 Collabora Ltd.\n"
+         "Copyright \xc2\xa9 2010 Andreas Henriksson"),
       _("A map viewer for Maemo 5.\n\nThe maps are from the OpenStreetMap project and\nmapsforfree.com.\n\nPlace search is provided by geonames.org.\n\nBusinesses search is powered by Praized Media.\nUS data provided by Localeze.\nCanadian business listings distributed by YellowPages.ca™"));
 
   label = gtk_label_new (NULL);
@@ -650,6 +697,82 @@ on_copy_link (GtkButton *button,
   gtk_clipboard_set_text (clipboard, url, -1);
 }
 
+static gchar *
+ask_user_for_gpx_export_uri (MapBuddyWindow *window)
+{
+  GtkWidget *dialog;
+  gchar *uri = NULL;
+  GTimeVal current_time;
+  int buflen;
+  gchar *suggested_file_name;
+
+  g_return_val_if_fail (MAP_BUDDY_IS_WINDOW (window), NULL);
+
+  /* make sure tracking is enabled */
+  g_return_val_if_fail (window->tracking != NULL, NULL);
+
+  /* make sure we have any location fixes to actually save. */
+  g_return_val_if_fail (
+      map_buddy_tracking_has_location_fixes(window->tracking), NULL);
+
+  /* ask user for location to save GPX file. */
+  dialog = hildon_file_chooser_dialog_new (GTK_WINDOW (window),
+      GTK_FILE_CHOOSER_ACTION_SAVE);
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog),
+      "/home/user/MyDocs");
+  gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog),
+      TRUE);
+
+  /* create a suggested filename containing current date. */
+  g_get_current_time (&current_time);
+  suggested_file_name = g_time_val_to_iso8601 (&current_time);
+  buflen = strlen (suggested_file_name);
+  /* YYYY-MM-DDTHH:MM:SS.... -> YYYY-MM-DD.gpx */
+  g_strlcpy(suggested_file_name + 10, ".gpx", buflen-10);
+  gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog),
+      suggested_file_name);
+  g_free (suggested_file_name);
+  hildon_file_chooser_dialog_focus_to_input (
+      HILDON_FILE_CHOOSER_DIALOG (dialog));
+
+  if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
+  {
+    uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+  }
+
+  gtk_widget_destroy (dialog);
+
+  return uri;
+}
+
+
+static void
+on_toggle_tracking (GtkButton *button,
+              MapBuddyWindow *self)
+{
+  if (self->tracking == NULL)
+  {
+    self->tracking = map_buddy_tracking_new ();
+    map_buddy_tracking_start (self->tracking, self->view, self->gps_control);
+    hildon_button_set_text (HILDON_BUTTON (self->tracking_item),
+        _("Disable Tracking"), NULL);
+  }
+  else
+  {
+    gchar *uri = ask_user_for_gpx_export_uri (self);
+    if (uri)
+    {
+      map_buddy_tracking_export_gpx(self->tracking, uri);
+      g_free (uri);
+    }
+
+    map_buddy_tracking_stop (self->tracking, self->view, self->gps_control);
+    g_object_unref (self->tracking);
+    self->tracking = NULL;
+    hildon_button_set_text (HILDON_BUTTON (self->tracking_item),
+        _("Enable Tracking"), NULL);
+  }
+}
 
 static gboolean
 on_key_pressed (GtkWindow* window,
@@ -739,11 +862,125 @@ on_layer (GtkButton *button,
   gtk_widget_hide (dialog);
 }
 
+static gboolean
+can_show_pois (MapBuddyWindow *self)
+{
+  gboolean ret = FALSE;
+
+  if (self->show_pois_set && champlain_view_get_zoom_level (self->view) > 15)
+    ret = TRUE;
+
+  return ret;
+}
+
+static void
+on_poi_osm_data_fetched (MapBuddyPoiOsmFetcher *fetcher,
+    const GError **error,
+    gpointer user_data)
+{
+  GtkWidget *dialog = GTK_WIDGET (user_data);
+
+  g_return_if_fail (GTK_IS_WIDGET (dialog));
+
+  gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+static void
+on_get_pois_dialog_response (GtkDialog *dialog,
+    gint response_id,
+    gpointer user_data)
+{
+  MapBuddyWindow *self = MAP_BUDDY_WINDOW (user_data);
+  GtkWidget *dialog_ok;
+  GtkWidget *label;
+  MapBuddyPoiOsmFetcher *fetcher;
+  gdouble top, buttom, left, right;
+  gfloat w, h;
+
+  g_return_if_fail (MAP_BUDDY_IS_WINDOW (self));
+
+  if (response_id != GTK_RESPONSE_ACCEPT)
+    goto out;
+
+  dialog_ok = hildon_dialog_new_with_buttons (_("Download POIs"),
+      GTK_WINDOW (self), GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+      NULL, NULL);
+
+  label = gtk_label_new (NULL);
+  gtk_label_set_markup (GTK_LABEL (label),
+      "Downloading, please be patient :-)\n"
+      "<b>It might take several minutes</b>, "
+      "depending on the size of the area.");
+  gtk_container_add (
+      GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog_ok))),
+      label);
+
+  gtk_widget_hide_all (GTK_WIDGET (dialog));
+  gtk_widget_show_all (dialog_ok);
+
+  /* get the geo coord for the current shown screen */
+  clutter_actor_get_size (CLUTTER_ACTOR (self->view), &w, &h);
+  champlain_view_get_coords_at (self->view, 0, 0, &top, &left);
+  champlain_view_get_coords_at (self->view, w, h, &buttom, &right);
+
+  /* fetch POIs */
+  fetcher = map_buddy_poi_manager_get_fetcher (self->poi_manager);
+
+  map_buddy_poi_osm_fetcher_get (fetcher, left, buttom, right, top,
+      on_poi_osm_data_fetched, dialog_ok);
+
+out:
+  gtk_widget_destroy (GTK_WIDGET (dialog));
+}
+
+static void
+on_get_pois (GtkButton *button,
+    MapBuddyWindow *self)
+{
+  GtkWidget *dialog;
+  GtkWidget *label;
+
+  dialog = hildon_dialog_new_with_buttons (_("Download POIs"),
+      GTK_WINDOW (self), GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+      GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
+
+  g_signal_connect (dialog, "response",
+      G_CALLBACK (on_get_pois_dialog_response), self);
+
+  label = gtk_label_new (NULL);
+  gtk_label_set_markup (GTK_LABEL (label),
+      "Download POIs for the shown map area?\n"
+      "<b>Download might take long</b>,\ndepending on the size of the area");
+  gtk_container_add (
+      GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
+      label);
+
+  gtk_widget_show_all (dialog);
+}
+
+static void
+on_show_pois (GtkButton *button,
+    MapBuddyWindow *self)
+{
+  if (self->show_pois_set)
+    {
+      self->show_pois_set = FALSE;
+      champlain_layer_hide (self->champlain_poi_layer);
+      hildon_button_set_title (HILDON_BUTTON (button), _("Show POIs"));
+    }
+  else
+    {
+      self->show_pois_set = TRUE;
+      if (can_show_pois (self))
+          champlain_layer_show (self->champlain_poi_layer);
+      hildon_button_set_title (HILDON_BUTTON (button), _("Hide POIs"));
+    }
+}
+
 static void
 on_show_contacts (GtkButton *button,
                   MapBuddyWindow *self)
 {
-
   if (CLUTTER_ACTOR_IS_VISIBLE (CLUTTER_ACTOR (self->contact_layer)))
     {
       champlain_layer_hide (self->contact_layer);
@@ -801,6 +1038,24 @@ on_location_error (LocationGPSDControl *control,
   self->has_fix = FALSE;
 }
 
+static void
+update_poi_layer_markers (MapBuddyWindow *self)
+{
+  gint zoom;
+
+  g_object_get (self->view,
+      "zoom-level", &zoom,
+      NULL);
+  g_print ("%s: to %d\n", G_STRFUNC, zoom);
+
+  /* don't bother showing POIs if the zoom level is too small, they probably
+   * would overlap */
+  if (can_show_pois (self))
+    champlain_layer_show (self->champlain_poi_layer);
+  else
+    champlain_layer_hide (self->champlain_poi_layer);
+}
+
 static void
 update_me_marker_radius (MapBuddyWindow *self)
 {
@@ -842,6 +1097,7 @@ on_zoom_level_changed (GObject *gobject,
                        MapBuddyWindow *self)
 {
   update_me_marker_radius (self);
+  update_poi_layer_markers (self);
 }
 
 
@@ -869,6 +1125,9 @@ on_location_changed (LocationGPSDevice *device,
       self->pos_error = device->fix->eph / 100.0;
 
       update_me_marker_radius (self);
+      if (map_buddy_tracking_record_location (self->tracking, device))
+        champlain_view_center_on (self->view, self->latitude, self->longitude);
+
       clutter_actor_show (CLUTTER_ACTOR (self->me_marker));
     }
   else
@@ -1139,7 +1398,7 @@ open_url (GtkButton* button,
     {
       /* Just do demonstrate remote exceptions versus regular GError */
       if (error->domain == DBUS_GERROR && error->code == DBUS_GERROR_REMOTE_EXCEPTION)
-        g_printerr ("Caught remote method exception %s: %s",
+        g_printerr ("Caught remote method exception %s: %s\n",
                     dbus_g_error_get_name (error),
                     error->message);
       else
@@ -1214,7 +1473,6 @@ place_call (GtkButton* button,
   DBusGProxy *proxy;
   gchar *stripped;
   GRegex *reg = g_regex_new ("[\\(\\)\\ \\-]", 0, 0, NULL);
-  gchar *phone;
 
   stripped = g_regex_replace (reg, number, -1, 0, "", 0, NULL);
 
@@ -1245,7 +1503,7 @@ place_call (GtkButton* button,
     {
       /* Just do demonstrate remote exceptions versus regular GError */
       if (error->domain == DBUS_GERROR && error->code == DBUS_GERROR_REMOTE_EXCEPTION)
-        g_printerr ("Caught remote method exception %s: %s",
+        g_printerr ("Caught remote method exception %s: %s\n",
                     dbus_g_error_get_name (error),
                     error->message);
       else
