@@ -63,10 +63,11 @@ map_buddy_me_marker_class_init (MapBuddyMeMarkerClass *map_buddyClass)
 
 static void
 draw_box (cairo_t *cr,
-   gint radius)
+    gint radius)
 {
-      cairo_arc (cr, radius, radius, radius -1, 0, 2 * M_PI);
-      cairo_close_path (cr);
+  /* M_PI is not defined in C99, so instead of using the constant, comp it */
+  cairo_arc (cr, radius, radius, radius -1, 0, 2 * acos (-1.0));
+  cairo_close_path (cr);
 }
 
 static void
